@@ -80,7 +80,7 @@ class SQLAlchemyProvider(VocabularyProvider):
         if thing.type and thing.type == 'collection':
             return Collection(
                 id=thing.concept_id,
-                uri=thing.uri,
+                uri=thing.uri if thing.uri is not None else self.uri_generator.generate(type='collection', id=thing.concept_id),
                 labels=[
                     Label(l.label, l.labeltype_id, l.language_id)
                     for l in thing.labels
@@ -91,7 +91,7 @@ class SQLAlchemyProvider(VocabularyProvider):
         else:
             return Concept(
                 id=thing.concept_id,
-                uri=thing.uri,
+                uri=thing.uri if thing.uri is not None else self.uri_generator.generate(type='concept', id=thing.concept_id),
                 labels=[
                     Label(l.label, l.labeltype_id, l.language_id)
                     for l in thing.labels
@@ -119,6 +119,18 @@ class SQLAlchemyProvider(VocabularyProvider):
         return self._from_thing(thing)
 
     def get_by_uri(self, uri):
+        '''Get all information on a concept or collection, based on a
+        :term:`URI`.
+
+        This method will only find concepts or collections whose :term:`URI` is 
+        actually stored in the database. It will not find anything that has
+        no :term:`URI` in the database, but does have a matching :term:`URI`
+        after generation.
+
+        :rtype: :class:`skosprovider.skos.Concept` or
+            :class:`skosprovider.skos.Collection` or `False` if the concept or
+            collection is unknown to the provider.
+        '''
         try:
             thing = self.session\
                         .query(Thing)\
@@ -238,8 +250,9 @@ class SQLAlchemyProvider(VocabularyProvider):
                 return False
 
             ids = self.session\
-                      .query(Visitation.concept_id)\
-                      .filter(Visitation.conceptscheme_id == self.conceptscheme_id)\
+                      .query(Thing.concept_id)\
+                      .join(Visitation)\
+                      .filter(Thing.conceptscheme_id == self.conceptscheme_id)\
                       .filter(Visitation.lft.between(cov[0], cov[1]))\
                       .all()
             ret = [id[0] for id in ids]
