@@ -32,7 +32,7 @@ def import_provider(provider, conceptscheme, session):
 
     '''
 
-    #First pass: load all concepts and collections
+    # First pass: load all concepts and collections
     for stuff in provider.get_all():
         c = provider.get_by_id(stuff['id'])
         if isinstance(c, Concept):
@@ -63,7 +63,12 @@ def import_provider(provider, conceptscheme, session):
 
     session.flush()
 
-    #Second pass: link
+    # Second pass: link
+    # Map for narrower concepts and collections
+    modelmap = {
+            'narrower_concepts': ConceptModel,
+            'narrower_collections': CollectionModel
+    }
     for stuff in provider.get_all():
         c = provider.get_by_id(stuff['id'])
         if isinstance(c, Concept):
@@ -73,11 +78,14 @@ def import_provider(provider, conceptscheme, session):
                             .filter(ConceptModel.concept_id == int(c.id))\
                             .one()
                 for nc in c.narrower:
-                    nc = session.query(ConceptModel)\
-                                .filter(ConceptModel.conceptscheme_id == conceptscheme.id)\
-                                .filter(ConceptModel.concept_id == int(nc))\
-                                .one()
-                    cm.narrower_concepts.add(nc)
+                    for narrow_model, Model in modelmap.items():
+                        nc_query = session.query(Model) \
+                            .filter(Model.conceptscheme_id == conceptscheme.id) \
+                            .filter(Model.concept_id == int(nc))
+                        if session.query(nc_query.exists()).one()[0]:
+                            nc = nc_query.one()
+                            getattr(cm, narrow_model).add(nc)
+                            break
             if len(c.related) > 0:
                 cm = session.query(ConceptModel)\
                             .filter(ConceptModel.conceptscheme_id == conceptscheme.id)\
